@@ -2,8 +2,8 @@
 ### Fit a ordinal linear mixed model with a Dirichlet Process prior
 ### for the random effect distribution.
 ###
-### Copyright: Alejandro Jara Vallejos, 2006
-### Last modification: 07-07-2006.
+### Copyright: Alejandro Jara, 2006-2007
+### Last modification: 30-04-2007.
 ###
 ### This program is free software; you can redistribute it and/or modify
 ### it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
 ###
 ### The author's contact information:
 ###
-###      Alejandro Jara Vallejos
+###      Alejandro Jara
 ###      Biostatistical Centre
 ###      Katholieke Universiteit Leuven
 ###      U.Z. Sint-Rafal
@@ -43,46 +43,36 @@ function(fixed,
          mcmc,
          state,
          status, 
-         data,
+         data=sys.frame(sys.parent()),
          na.action=na.fail)
 {
-         #########################################################################################
-         # call parameters
-         #########################################################################################
+       #########################################################################################
+       # call parameters
+       #########################################################################################
          m <- mcall <- cl <- match.call()
          nm <- names(m)[-1]
-         keep <- is.element(nm, c("data", "na.action","offset"))
+         keep <- is.element(nm, c("data", "na.action"))
          for (i in nm[!keep]) m[[i]] <- NULL
-         cfixed<-all.vars(fixed)
-         crandom<-all.vars(random)
-         alist <- lapply(as.list(c(crandom,cfixed)), as.name)
-         osubject<-do.call('data.frame', alist[length(crandom)])
-         namessub<-names(table(osubject))
-         nosubject<-length(namessub)
-         idsubject<-rep(0,dim(osubject)[1])
- 
-         for(i in 1:nosubject){
-             idsubject[osubject==namessub[i]]<-i
-         }
- 
-         allvars<-NULL
-         if(length(cfixed) >1) allvars <-c(cfixed[2:length(cfixed)])
-         if(length(crandom)>1) allvars <-c(allvars,crandom[1:(length(crandom)-1)],"idsubject")
-         if(length(crandom)==1) allvars <-c(allvars,"idsubject")
          
-         m$formula <- as.formula(paste(cfixed[1],"~", paste(allvars, collapse = "+")))
+         allvars <- c(all.vars(fixed), all.vars(random))
+
+         Terms <- if (missing(data)) 
+              terms(fixed)
+         else terms(fixed, data = data)
+         
+         cl$fixed <- eval(fixed)
+         cl$random <- eval(random)
+         m$formula <- as.formula(paste("~", paste(allvars, collapse = "+")))
          environment(m$formula) <- environment(fixed)
          m$drop.unused.levels <- TRUE
-         Mdata<-do.call('data.frame', alist)
-         m$data<-data.frame(idsubject,Mdata)
          m[[1]] <- as.name("model.frame")
          mf <- eval.parent(m)
-         
-         #########################################################################################
-         # data structure
-         #########################################################################################
-     	 resp<-model.response(mf,"numeric")
 
+       #########################################################################################
+       # data structure
+       #########################################################################################
+         nrec <- dim(mf)[1]
+         resp <- mf[,1]
          if(!is.factor(resp)) stop("the response must be a factor")
          categ<-levels(resp)
          ncateg<-length(categ)
@@ -92,140 +82,255 @@ function(fixed,
          if(min(yr)>1)yr<-yr+min(yr)+1
          categn<-seq(1,ncateg)
 
-	 nrec<-length(resp)
-	 
-	 mat<-model.matrix(m$formula,mf)
-	 pp<-dim(mat)[2]
-        
-         #random effects
-         idrec<-seq(1,nrec)
-         namessub2<-names(table(mf$idsubject))
-         nsubject<-length(namessub2)
-         namesre<-rep(0,nsubject)
-         newid<-rep(0,nrec)
-         for(i in 1:nsubject){
-             newid[mf$idsubject==namessub2[i]]<-i
-             namesre[i]<-namessub[as.integer(namessub2[i])]    
+         crandom <- all.vars(random)
+         namesre <- (names(mf)==crandom[length(crandom)])
+         oldid <- mf[,namesre]
+         freqsub<-table(oldid)
+         namesre <- names(freqsub)         
+         nsubject <- length(namesre)
+         newid <- seq(1,nrec)
+         for(i in 1:nsubject)
+         {
+             newid[oldid==namesre[i]] <- i
          }
-         mf$idsubject<-newid
-
-         freqsub<-table(mf$idsubject)
-         maxni<-max(freqsub)
-         datastr<-matrix(0,nrow=nsubject,ncol=maxni+1)
-         datastr[,1]<-freqsub
-         for(i in 1:nsubject){
-             for(j in 1:freqsub[i]){
-                datastr[i,(j+1)]<-idrec[idsubject==i][j] 
+         
+         maxni <- max(freqsub)
+         idrec <- seq(1,nrec)
+         datastr <- matrix(0,nrow=nsubject,ncol=maxni+1)
+         datastr[,1] <- freqsub
+         for(i in 1:nsubject)
+         {
+             for(j in 1:freqsub[i])
+             {
+                 datastr[i,(j+1)] <- idrec[newid==i][j] 
              }
          }
 
-         #########################################################################################
-         # design matrix for random and fixed effects
-         #########################################################################################
-         q<-1
-         z<-mat[,1]
-         possr<-1
-         if(length(crandom)>1){
-            for(i in 1:(length(crandom)-1)){
-               for(j in 1:(pp-1)){
-                  if(crandom[i]==colnames(mat)[j]){
-                     z<-cbind(z,mat[,j])
-                     q<-q+1
-                     possr<-c(possr,j)
-                  }   
-               }
+       #########################################################################################
+       # model structure
+       #########################################################################################
+         q <- length(crandom)
+         z<-matrix(1,nrow=nrec,ncol=1)
+         colnames(z) <- "(Intercept)"
+         nvarrand <- "(Intercept)"
+
+         if(q>1)
+         {
+            for(i in 1:(q-1))
+            {
+               zwork <- (names(mf)==crandom[i])
+               zwork <- matrix(mf[,zwork],nrow=nrec,ncol=1)
+               colnames(zwork) <- crandom[i]
+               nvarrand <- c(nvarrand,crandom[i])
+               z <- cbind(z,zwork)
             }
+         }   
+         
+         cfixed <- all.vars(fixed)
+         cfixed <- cfixed[-1]
+
+         for(i in 1:q)
+         {
+            if(sum(nvarrand[i]==cfixed) != 0)
+            {
+               stop("Covariates cannot be included in the random and fixed part of the model")
+            }   
+         }   
+         x <- model.matrix(fixed,data=mf)
+         p <- dim(x)[2]
+         x <- x[,-1]
+         p <- p-1
+         nfixed <- p
+         
+         if(p==0)
+         {
+            nfixed <- 0
+            p <- 1
+            x <- matrix(0,nrow=nrec,ncol=1)
          }
-         if(q==1)z<-matrix(z,ncol=1)
-         colnames(z)<-colnames(mat)[possr]
-
-         x<-NULL
-         possf<-NULL
-         nfixed<-0
-         for(i in 1:(pp-1)){
-            type<-0
-            for(j in 1:q){
-               if(i==possr[j])type<-1
-            }
-            if(type==0){
-              x<-cbind(x,mat[,i])   
-              nfixed<-nfixed+1
-              possf<-c(possf,i)
-            }  
-         }
-
-         #########################################################
-         #### NOTE: Is there is no fixed effects in the model ####
-         ####       nfixed=0 and p=1.                         ####
-         #########################################################
-
-         if(nfixed==0){
-            p<-1
-            x<-matrix(0,nrow=nrec,ncol=1)
-         }
-
-         if(nfixed>0){
-            p<-nfixed
-            colnames(x)<-colnames(mat)[possf]
-         }
-
          xtx<-t(x)%*%x
+         
+       #########################################################################################
+       # elements for Pseudo Countour Probabilities' computation
+       #########################################################################################
+         possiP <- NULL
+         if(nfixed>0)
+         {
+            mat <- attr(Terms,"factors")
+            namfact <- colnames(mat)
+            nvar <- dim(mat)[1]
+            nfact <- dim(mat)[2]
+            possiP <- matrix(0,ncol=2,nrow=nfact)
+            if (missing(data)) dataF <- model.frame(formula=fixed,xlev=NULL)
+               dataF <- model.frame(formula=fixed,data,xlev=NULL)
+            namD <- names(dataF)
+            isF <- sapply(dataF, function(x) is.factor(x) || is.logical(x))
+            nlevel <- rep(0,nvar)
+            for(i in 1:nvar)
+            {
+                if(isF[i])
+                {
+                   nlevel[i]<-length(table(dataF[[i]]))
+                }
+                else
+                {
+                   nlevel[i]<-1
+                }
+            }
+            startp<-1+q
+            for(i in 1:nfact)
+            {
+                tmp1<-1
+                for(j in 1:nvar)
+                {
+                    if(mat[j,i]==1 && isF[j])
+                    {
+                       tmp1<-tmp1*(nlevel[j]-1)
+                    }
+                }
+                endp<-startp+tmp1-1
+                possiP[i,1]<-startp    
+                possiP[i,2]<-endp
+                startp<-endp+1
+            }
+            dimnames(possiP)<-list(namfact,c("Start","End"))
+         }   
 
-         #########################################################################################
-         # prior information
-         #########################################################################################
-  	 
-  	 if(is.null(prior$a0))
-  	 {
-  	    a0b0<-c(-1,-1)
-  	    alpha<-prior$alpha
-  	    alphapr<-0
-  	 }
+       #########################################################################################
+       # prior information
+       #########################################################################################
+         if(nfixed==0)
+         {
+            prec<-matrix(0,nrow=1,ncol=1)
+            sb<-matrix(0,nrow=1,ncol=1)
+            b0<-rep(0,1)
+         }
+         else
+         {
+            b0<-prior$beta0
+            prec<-solve(prior$Sbeta0)
+            sb<-prec%*%b0
+
+            if(length(b0)!=p)
+            { 
+                   stop("Error in the dimension of the mean of the normal prior for the fixed effects.\n")     
+            }
+
+            if(dim(prec)[1]!=p || dim(prec)[2]!=p)
+            { 
+                   stop("Error in the dimension of the covariance of the normal prior for the fixed effects.\n")     
+            }
+
+         }
+
+         if(is.null(prior$a0))
+         {
+            a0b0<-c(-10,-10)
+            alpha<-prior$alpha
+            alphapr<-0
+         }
          else
          {
             a0b0<-c(prior$a0,prior$b0)
-  	    alpha<-rgamma(1,shape=prior$a0,scale=prior$b0)
-  	    alphapr<-1
-  	 }
-  	 
-  	 nu0<-prior$nu0
-  	 
-	 tinv<-prior$tinv
-	 
-	 if(nfixed==0){
-	    prec<-matrix(0,nrow=1,ncol=1)
-	    sb<-matrix(0,nrow=1,ncol=1)
-	 }
-
-	 if(nfixed>0){
-            prec<-solve(prior$Sbeta0)
-            sb<-prec%*%prior$beta0
+            alpha<-rgamma(1,shape=prior$a0,scale=prior$b0)
+            alphapr<-1
+            if(prior$a0<0 || prior$b0<0)
+            { 
+                   stop("The parameters of the Gamma prior for the precision parameter must be possitive.\n")     
+            }
          }
 
-	 psiinv<-solve(prior$Sb)
-	 smu<-psiinv%*%prior$mub
-	 
-	 
-         #########################################################################################
-         # mcmc specification
-         #########################################################################################
-         mcmcvec<-c(mcmc$nburn,mcmc$nskip,mcmc$ndisplay)
-         nsave<-mcmc$nsave
+  	 if(is.null(prior$mub))
+  	 {
+  	    murand<-0 
+            if(is.null(prior$mu))
+            { 
+               stop("The vector *mu* must be specified in the prior object when it is not considered as random.\n")     
+            }
+            if(length(prior$mu) != q)
+            { 
+               stop("Error in the dimension of the mean the centering distribution.\n")     
+            }
+            psiinv<-diag(1,q)
+            smu<-rep(0,q)
+         }
+         else
+         {
+            murand<-1
+  	    psiinv<-solve(prior$Sb)
+	    smu<-psiinv%*%prior$mub
+            if(length(prior$mub) != q)
+            { 
+               stop("Error in the dimension of the mean of the normal prior for the mean of the centering distribution.\n")     
+            }
 
-         #########################################################################################
-         # output
-         #########################################################################################
+            if(is.null(dim(psiinv)) && q==1) 
+            {
+               stop("Error in the dimension of the covariance of the normal prior for the mean of the centering distribution.\n")     
+            }   
+
+            if(!is.null(dim(psiinv)) && ( dim(psiinv)[1]!=q || dim(psiinv)[2]!=q ))
+            { 
+               stop("Error in the dimension of the covariance of the normal prior for the mean of the centering distribution.\n")     
+            }
+         }
+
+
+  	 if(is.null(prior$nu0))
+  	 {
+            sigmarand<-0
+            if(is.null(prior$sigma))
+            { 
+               stop("The matrix *sigma* must be specified in the prior object when it is not considered as random.\n")     
+            }
+            nu0 <- -1
+            tinv<-diag(1,q)
+  	 }
+  	 else
+  	 {
+  	   sigmarand<-1
+           nu0<-prior$nu0
+           if(nu0<=0)
+           { 
+                stop("The parameter of the IW prior distribution must be positive")     
+           }
+
+           tinv<-prior$tinv
+           if(dim(tinv)[1]!=q || dim(tinv)[2]!=q)
+           { 
+                stop("Error in the dimension of the matrix of the IW prior for the covariance of the centering distribution.\n")     
+           }
+         }
+
+       #########################################################################################
+       # mcmc specification
+       #########################################################################################
+         if(missing(mcmc))
+         {
+            nburn <- 1000
+            nsave <- 1000
+            nskip <- 0
+            ndisplay <- 100
+            mcmcvec<-c(nburn,nskip,ndisplay,murand,sigmarand)
+         }
+         else
+         {
+            mcmcvec<-c(mcmc$nburn,mcmc$nskip,mcmc$ndisplay,murand,sigmarand)
+            nsave<-mcmc$nsave
+         }
+
+       #########################################################################################
+       # output
+       #########################################################################################
          nuniq<-(q*(q+1)/2)
-         
+         mc<-rep(0,5)         
          randsave<-matrix(0,nrow=nsave,ncol=q*(nsubject+1))
          thetasave<-matrix(0,nrow=nsave,ncol=q+nfixed+q+nuniq+2+ncateg-2)
-         
-         cpo<-rep(0,nrec)
+         cpo<-matrix(0,nrow=nrec,ncol=2)
 
-
-         #########################################################################################
-         # MLE estimation
-         #########################################################################################
+       #########################################################################################
+       # MLE estimation
+       #########################################################################################
          
          MLEordprobit<-function(x,y,p,ncateg,nrec)
          {
@@ -268,50 +373,94 @@ function(fixed,
 	     return(out)
          }
 
-         #########################################################################################
-         # parameters depending on status
-         #########################################################################################
+       #########################################################################################
+       # parameters depending on status
+       #########################################################################################
 
     	 if(status==TRUE)
 	 {
-	        if(nfixed==0){
+
+	        if(sigmarand==1)
+	        {
+                   wsigma <- prior$tinv/(prior$nu0-q-1)
+                }
+                else
+                {
+                   wsigma <- prior$sigma
+                }
+
+                bzs<-NULL
+                for(i in 1:q)
+                {
+                   work<-rnorm(nsubject,mean=0,sd=sqrt(wsigma[i,i]))
+                   bzs<-cbind(bzs,work)
+                }
+	 
+	        if(nfixed==0)
+	        {
 	           beta<-matrix(0,nrow=1,ncol=1)
-                   
                    fit0<-MLEordprobit(z,yr,q,ncateg,nrec)
-                  
 	           b<-matrix(0,nrow=nsubject,ncol=q)
 		   bclus<-matrix(0,nrow=nsubject,ncol=q)
 		   
-	           bclus[1,]<-fit0$beta[1:q]
-	           for(i in 1:nsubject){
-	               b[i,]<-fit0$beta[1:q]
+	           for(i in 1:nsubject)
+	           {
+	               b[i,]<-fit0$beta[1:q]+bzs[i,]
 	           }
-	           mu<-fit0$beta[1:q]
-	           sigma<- fit0$covb[1:q,1:q]
+	           bclus<-b
+	           
+	           if(murand==1)
+	           {
+  	               mu<-fit0$beta[1:q]
+  	           }
+  	           else
+  	           {
+  	               mu<-prior$mu
+  	           }
+  	           
+	           if(sigmarand==1)
+	           {
+  	               sigma<-prior$tinv/(prior$nu0-q-1)
+  	           }
+  	           else
+  	           {
+  	               sigma<-prior$sigma
+  	           }
 	           sigmainv<-solve(sigma)
-
 		   cutoff<-c(0,fit0$beta[(q+1):(q+ncateg-2)])
 		   cutoff<-cumsum(c(cutoff[1], exp(cutoff[-1])))
 	           
 	        }
 
-	        if(nfixed>0){
-
+	        if(nfixed>0)
+	        {
                    fit0<-MLEordprobit(cbind(x,z),yr,(p+q),ncateg,nrec)
-
 	           b<-matrix(0,nrow=nsubject,ncol=q)
 		   bclus<-matrix(0,nrow=nsubject,ncol=q)
-
                    beta<-fit0$beta[1:p]
-		   bclus[1,]<-fit0$beta[(p+1):(p+q)]
-
 	           for(i in 1:nsubject){
-	               b[i,]<-fit0$beta[(p+1):(p+q)]
+	               b[i,]<-fit0$beta[(p+1):(p+q)]+bzs[i,]
 	           }
-	           mu<-fit0$beta[(p+1):(p+q)]
-	           sigma<- fit0$covb[(p+1):(p+q),(p+1):(p+q)]
+	           bclus<-b
+	           
+	           if(murand==1)
+	           {
+  	               mu<-fit0$beta[(p+1):(p+q)]
+  	           }
+  	           else
+  	           {
+  	               mu<-prior$mu
+  	           }
+  	           
+	           if(sigmarand==1)
+	           {
+  	               sigma<-prior$tinv/(prior$nu0-q-1)
+  	           }
+  	           else
+  	           {
+  	               sigma<-prior$sigma
+  	           }
 		   sigmainv<-solve(sigma)
-
 		   cutoff<-c(0,fit0$beta[(p+q+1):(p+q+ncateg-2)])
 		   cutoff<-cumsum(c(cutoff[1], exp(cutoff[-1])))
 	        }
@@ -325,21 +474,29 @@ function(fixed,
 	        alpha<-state$alpha
                 b<-state$b 
                 bclus<-state$bclus 
-	        beta<-state$beta
-	        betar<-state$betar
+                if(nfixed>0)
+                {
+	           beta<-state$beta
+	        }
+	        else
+	        {
+	           beta<-rep(0,p)
+	        }
 	        cutoff<-state$cutoff
 	        mu<-state$mu
 	        ncluster<-state$ncluster
 	        sigma<-state$sigma
 	        sigmainv<-solve(sigma)
 	        ss<-state$ss
+	        betar<-rep(0,q)
 	 }
 
 
-         #########################################################################################
-         # working space
-         #########################################################################################
+       #########################################################################################
+       # working space
+       #########################################################################################
          ccluster<-rep(0,nsubject) 
+         cstrt<-matrix(0,nrow=nsubject,ncol=nsubject)          
          iflag<-rep(0,p) 
          iflag2<-rep(0,maxni)
          iflagb<-rep(0,q) 
@@ -350,7 +507,6 @@ function(fixed,
          seed<-c(seed1,seed2)
          theta<-rep(0,q)
          work1<-matrix(0,nrow=p,ncol=p)
-         work2<-matrix(0,nrow=p,ncol=p)
          workb1<-matrix(0,nrow=q,ncol=q)
          workb2<-matrix(0,nrow=q,ncol=q)
          workmh1<-rep(0,p*(p+1)/2) 
@@ -361,20 +517,20 @@ function(fixed,
          workkm1<-matrix(0,nrow=maxni,ncol=maxni) 
          workkm2<-matrix(0,nrow=maxni,ncol=maxni) 
          workv1<-rep(0,p) 
-         workv2<-rep(0,p) 
          workvb1<-rep(0,q) 
-         workvb2<-rep(0,q) 
          xty<-rep(0,p) 
          y<-rep(0,nrec) 
          ywork<-rep(0,maxni) 
          zty<-rep(0,q) 
          ztz<-matrix(0,nrow=q,ncol=q) 
-         ztzinv<-matrix(0,nrow=q,ncol=q) 
+
+         betasave<-rep(0,(p+ncateg-1))
+         bsave<-matrix(0,nrow=nsubject,ncol=q)
          
-         #########################################################################################
-         # calling the fortran code
-         #########################################################################################
-         foo <- .Fortran("spolmmp",
+       #########################################################################################
+       # calling the fortran code
+       #########################################################################################
+         foo <- .Fortran("dpolmmp",
          	datastr    =as.integer(datastr),
  	 	maxni      =as.integer(maxni),         
  	 	ncateg     =as.integer(ncateg),         
@@ -411,7 +567,9 @@ function(fixed,
  		ncluster   =as.integer(ncluster),
  		sigma      =as.double(sigma),
  		ss         =as.integer(ss),
+ 		mc         =as.double(mc), 		
  		ccluster   =as.integer(ccluster),
+ 		cstrt      =as.integer(cstrt),
  		iflag      =as.integer(iflag),
  		iflag2     =as.integer(iflag2),
  		iflagb     =as.integer(iflagb),
@@ -421,7 +579,6 @@ function(fixed,
  		sigmainv   =as.double(sigmainv),
  		theta      =as.double(theta),
  		work1      =as.double(work1),
- 		work2      =as.double(work2),
  		workb1     =as.double(workb1),
  		workb2     =as.double(workb2),
                 workmh1    =as.double(workmh1),
@@ -432,83 +589,119 @@ function(fixed,
  		workkm1    =as.double(workkm1),
  		workkm2    =as.double(workkm2),
  		workv1     =as.double(workv1),
- 		workv2     =as.double(workv2),
  		workvb1    =as.double(workvb1),
- 		workvb2    =as.double(workvb2),
  		xty        =as.double(xty),
  		ywork      =as.double(ywork),
  		zty        =as.double(zty), 		
- 		ztz        =as.double(ztz), 		
- 		ztzinv     =as.double(ztzinv), 		
+ 		ztz        =as.double(ztz), 
+                betasave   =as.double(betasave),
+                bsave      =as.double(bsave),
 		PACKAGE    ="DPpackage")	
 
-         #########################################################################################
-         # save state
-         #########################################################################################
+       #########################################################################################
+       # save state
+       #########################################################################################
 
          dimen<-q+nfixed+q+nuniq+2+ncateg-2
+         mc<-foo$mc
+         names(mc)<-c("Dbar", "Dhat", "pD", "DIC","LPML")
          thetasave<-matrix(foo$thetasave,nrow=nsave, ncol=dimen)
          randsave<-matrix(foo$randsave,nrow=nsave, ncol=q*(nsubject+1))
-         cpo<-foo$cpo
+
+         cpom<-matrix(foo$cpo,nrow=nrec,ncol=2)
+         cpo<-cpom[,1]         
+         fso<-cpom[,2]
  
- 	 if(nfixed==0)pnames1<- c(colnames(mat)[possr])
- 	 if(nfixed >0)pnames1<- c(colnames(mat)[possr],colnames(mat)[possf])
+         if(nfixed==0)
+         {
+            pnames1 <- colnames(z)
+         }
+         if(nfixed==1)
+         {
+            pnames1 <- c(colnames(z),cfixed)
+         }
+         if(nfixed>1)
+         {
+            pnames1 <- c(colnames(z),colnames(x))
+         }   
  	 
- 	 pnames3<- paste("mu",colnames(mat)[possr],sep="-")
-         pnames4<-NULL
-         
-         for(i in 1:q){
-            for(j in i:q){
-               if(i==j)aa<-paste("sigma",colnames(mat)[possr][i],sep="-")
-               if(i!=j)aa<-paste("sigma",colnames(mat)[possr][i],colnames(mat)[possr][j],sep="-")
-               pnames4<-c(pnames4,aa)            
+         pnames2 <- paste("mu",colnames(z),sep="-") 	     	 
+         pnames3<-NULL
+         for(i in 1:q)
+         {
+            for(j in i:q)
+            {
+               if(i==j) aa <-paste("sigma",colnames(z)[i],sep="-")
+               if(i!=j) aa <-paste("sigma",colnames(z)[i],colnames(z)[j],sep="-")
+               pnames3<-c(pnames3,aa)            
             }
          }
          
-         pnames5<-paste("cutoff",2:(ncateg-1))
+         pnames4<-paste("cutoff",2:(ncateg-1))
          
-         pnames6<- c("ncluster","alpha")
+         pnames5<- c("ncluster","alpha")
          
-         colnames(thetasave)<-c(pnames1,pnames3,pnames4,pnames5,pnames6)
+         colnames(thetasave)<-c(pnames1,pnames2,pnames3,pnames4,pnames5)
          
-         qnames<-NULL
-         for(i in 1:nsubject){
-             for(j in 1:q){
-                 idname<-paste("(Subject",namesre[i],sep="=")
-                 idname<-paste(idname,")")
-                 qnamestemp<-paste(colnames(mat)[possr][j],idname,sep=" ")
-                 qnames<-c(qnames,qnamestemp)
+         qnames <- NULL
+         for(i in 1:nsubject)
+         {
+             for(j in 1:q)
+             {
+                 idname <- paste("(Subject",namesre[i],sep="=")
+                 idname <- paste(idname,")")
+                 qnamestemp <- paste(colnames(z)[j],idname,sep=" ")
+                 qnames <- c(qnames,qnamestemp)
              }
          }
-         for(j in 1:q){
-             qnamestemp<-paste("pred",colnames(mat)[possr][j],sep="-")
-             qnames<-c(qnames,qnamestemp)
+         for(j in 1:q)
+         {
+             qnamestemp <- paste("pred",colnames(z)[j],sep="-")
+             qnames <- c(qnames,qnamestemp)
          }
-         
-         colnames(randsave)<-qnames
-         
-         
+         colnames(randsave) <- qnames
+        
 	 model.name<-"Bayesian semiparametric ordinal linear mixed effect model"		
 
          coeff<-apply(thetasave, 2, mean)
 
-	 names(coeff)<-c(pnames1,pnames3,pnames4,pnames5,pnames6)
-
-
-	 state <- list(alpha=foo$alpha,b=matrix(foo$b,nrow=nsubject,ncol=q),
+	 state <- list(alpha=foo$alpha,
+	               b=matrix(foo$b,nrow=nsubject,ncol=q),
 	               bclus=matrix(foo$bclus,nrow=nsubject,ncol=q),
-	               beta=foo$beta,betar=foo$betar,
-	               mu=foo$mu,ncluster=foo$ncluster,
+	               beta=foo$beta,
+	               cutoff=foo$cutoff,
+	               mu=foo$mu,
+	               ncluster=foo$ncluster,
 	               sigma=matrix(foo$sigma,nrow=q,ncol=q),
 	               ss=foo$ss)
 
 	 save.state <- list(thetasave=thetasave,randsave=randsave)
 
-	 z<-list(modelname=model.name,coefficients=coeff,call=cl,
-                 prior=prior,mcmc=mcmc,state=state,save.state=save.state,nrec=foo$nrec,
-                 nsubject=foo$nsubject,nfixed=foo$nfixed,nrandom=foo$q,
-                 cpo=foo$cpo,alphapr=alphapr,namesre1=namesre,namesre2=colnames(mat)[possr],
-                 z=z,x=x,mf=mf,dimen=dimen,ncateg=ncateg,nsave=nsave)
+	 z<-list(modelname=model.name,
+	         coefficients=coeff,call=cl,
+                 prior=prior,
+                 mcmc=mcmc,
+                 state=state,
+                 save.state=save.state,
+                 nrec=foo$nrec,
+                 nsubject=foo$nsubject,
+                 nfixed=foo$nfixed,
+                 nrandom=foo$q,
+                 cpo=cpo,
+                 fso=fso,                                  
+                 alphapr=alphapr,
+                 namesre1=namesre,
+                 namesre2=colnames(z),
+                 z=z,
+                 x=x,
+                 mf=mf,
+                 dimen=dimen,
+                 ncateg=ncateg,
+                 nsave=nsave,
+                 y=resp,
+                 possiP=possiP,
+                 fixed=fixed,
+                 mc=mc)
                  
          cat("\n\n")        
 
@@ -518,10 +711,236 @@ function(fixed,
 
 
 ###
-### Tools for DPolmm: print, summary, plot
+### Tools for DPolmm: anova, print, summary, plot
 ###
-### Copyright: Alejandro Jara Vallejos, 2006
-### Last modification: 07-07-2006.
+### Copyright: Alejandro Jara, 2006-2007
+### Last modification: 01-04-2007.
+
+"anova.DPolmm"<-function(object, ...)
+{
+
+######################################################################################
+cregion<-function(x,probs=c(0.90,0.975))
+######################################################################################
+#  Function to compute a simultaneous credible region for a vector 
+#  parameter from the MCMC sample
+# 
+#  Reference: Besag, J., Green, P., Higdon, D. and Mengersen, K. (1995)
+#             Bayesian computation and stochastic systems (with Discussion)
+#             Statistical Science, vol. 10, 3 - 66, page 30
+#  and        Held, L. (2004) Simultaneous inference in risk assessment; a Bayesian 
+#             perspective In: COMPSTAT 2004, Proceedings in Computational 
+#             Statistics (J. Antoch, Ed.) 213 - 222, page 214
+#
+#  Arguments 
+#  sample : a data frame or matrix with sampled values (one column = one parameter).
+#  probs  : probabilities for which the credible regions are computed.
+######################################################################################
+{
+    #Basic information
+     nmonte<-dim(x)[1]
+     p<-dim(x)[2]
+     
+    #Ranks for each component
+     ranks <- apply(x, 2, rank, ties.method="first")
+     
+    #Compute the set S={max(nmonte+1-min r_i(t) , max r_i(t)): t=1,..,nmonte}
+     left <- nmonte + 1 - apply(ranks, 1, min)
+     right <- apply(ranks, 1, max)
+     S <- apply(cbind(left, right), 1, max)
+     S <- S[order(S)]
+    
+    #Compute the credible region
+     k <- floor(nmonte*probs)     
+     tstar <- S[k]
+     out<-list()
+     for(i in 1:length(tstar))
+     {
+        upelim <- x[ranks == tstar[i]]
+        lowlim <- x[ranks == nmonte + 1 - tstar[i]]    
+        out[[i]] <- rbind(lowlim, upelim)
+        rownames(out[[i]]) <- c("Lower", "Upper")
+        colnames(out[[i]]) <- colnames(x)
+     }
+     names(out) <- paste(probs)
+     return(out)
+}
+
+######################################################################################
+cint<-function(x,probs=c(0.90,0.975))
+######################################################################################
+#  Function to compute a credible interval from the MCMC sample
+#
+#  Arguments 
+#  sample : a data frame or matrix with sampled values (one column = one parameter).
+#  probs  : probabilities for which the credible regions are to be computed.
+######################################################################################
+{
+    #Compute the credible interval
+     delta<-(1-probs)/2
+     lprobs<-cbind(delta,probs+delta) 
+     out<-matrix(quantile(x,probs=lprobs),ncol=2)
+     colnames(out) <- c("Lower","Upper")
+     rownames(out) <- paste(probs)
+     return(out)
+}
+
+######################################################################################
+hnulleval<-function(mat,hnull)
+######################################################################################
+#  Evaluate H0
+#  AJV, 2006
+######################################################################################
+{
+     npar<-dim(mat)[2]   
+     lower<-rep(0,npar)
+     upper<-rep(0,npar)
+     for(i in 1:npar)
+     {
+        lower[i]<-mat[1,i]< hnull[i]
+        upper[i]<-mat[2,i]> hnull[i]
+     }
+     total<-lower+upper
+     out<-(sum(total==2) == npar)
+     return(out)
+}
+
+######################################################################################
+hnulleval2<-function(vec,hnull)
+######################################################################################
+#  Evaluate H0
+#  AJV, 2006
+######################################################################################
+{
+     lower<-vec[1]< hnull
+     upper<-vec[2]> hnull
+
+     total<-lower+upper
+     out<-(total==2)
+     return(out)
+}
+
+######################################################################################
+pcp<-function(x,hnull=NULL,precision=0.001,prob=0.95,digits=digits)
+######################################################################################
+#  Function to compute Pseudo Countour Probabilities (Region)
+#  AJV, 2006
+######################################################################################
+{
+    if(is.null(hnull))hnull<-rep(0,dim(x)[2])
+    if (dim(x)[2]!=length(hnull)) stop("Dimension of x and hnull must be equal!!")
+
+    probs <- seq(precision, 1-precision, by=precision)
+    neval <- length(probs)
+    probsf <- c(prob,probs)
+    cr <-  cregion(x,probs=probsf)
+
+    is.hnull <- hnulleval(cr[[2]],hnull)
+    if(is.hnull)
+    {
+       pval <- 1-precision
+    }   
+    else
+    {
+       is.hnull <- hnulleval(cr[[length(cr)]],hnull)
+       if (!is.hnull) 
+       {
+         pval <- precision
+       }  
+       else
+       {
+         is.hnull<-rep(0,neval+1)
+         for(i in 1:(neval+1))
+         {
+            is.hnull[i] <- hnulleval(cr[[i]],hnull)
+         }   
+         is.hnull <- is.hnull[-1]
+         first <- neval - sum(is.hnull) + 1
+         pval <- 1 - probs[first]
+       }
+    }
+    output <- list(cr=cr[[1]], prob=prob, pval=pval,hnull=hnull)
+    return(output)
+}
+
+
+######################################################################################
+pcp2<-function(x,hnull=NULL,precision=0.001,prob=0.95)
+######################################################################################
+#  Function to compute Pseudo Countour Probabilities (Interval)
+#  AJV, 2006
+######################################################################################
+{
+    if(is.null(hnull))hnull<-0
+    probs <- seq(precision, 1-precision, by=precision)
+    neval <- length(probs)
+    probsf <- c(prob,probs)
+    cr <-  cint(x,probs=probsf)
+
+    is.hnull <- hnulleval2(cr[2,],hnull)
+    if(is.hnull)
+    {
+       pval <- 1-precision
+    }   
+    else
+    {
+       is.hnull <- hnulleval2(cr[(neval+1),],hnull)
+       if (!is.hnull) 
+       {
+         pval <- precision
+       }  
+       else
+       {
+         is.hnull<-rep(0,neval+1)
+         for(i in 1:(neval+1))
+         {
+            is.hnull[i] <- hnulleval2(cr[i,],hnull)
+         }   
+         is.hnull <- is.hnull[-1]
+         first <- neval - sum(is.hnull) + 1
+         pval <- 1-probs[first]
+       }
+    }
+    output <- list(cr=cr[1,], prob=prob, pval=pval,hnull=hnull)
+    return(output)
+}
+
+######################################################################################
+######################################################################################
+######################################################################################
+    if(object$nfixed>0)
+    {
+       possiP<-object$possiP
+       nfact<-dim(possiP)[1]
+       P<-rep(0,nfact)
+       df<-rep(0,nfact)
+    
+       for(i in 1:nfact)
+       {
+           df[i]<-1
+           if((possiP[i,2]-possiP[i,1])>0)
+           { 
+              x<-matrix(object$save.state$thetasave[,possiP[i,1]:possiP[i,2]])
+              foo<-pcp(x=x) 
+              P[i]<-foo$pval
+              df[i]<-(possiP[i,2]-possiP[i,1])+1
+           }
+           else
+           {
+              x<-object$save.state$thetasave[,possiP[i,1]:possiP[i,2]]
+              foo<-pcp2(x=x) 
+              P[i]<-foo$pval
+           }
+       }
+
+       table <- data.frame(df,P) 
+       dimnames(table) <- list(rownames(possiP), c("Df","PsCP"))
+       structure(table, heading = c("Table of Pseudo Contour Probabilities\n", 
+        paste("Response:", deparse(formula(object$fixed)[[2]]))), class = c("anovaPsCP",
+        "data.frame"))
+    }    
+}
+
 
 
 "print.DPolmm"<-function (x, digits = max(3, getOption("digits") - 3), ...) 
@@ -616,12 +1035,6 @@ function(fixed,
          coef.u<-limm[2,]
     }
 
-    names(coef.m)<-names(object$coefficients[1:dimen1])
-    names(coef.sd)<-names(object$coefficients[1:dimen1])
-    names(coef.se)<-names(object$coefficients[1:dimen1])
-    names(coef.l)<-names(object$coefficients[1:dimen1])
-    names(coef.u)<-names(object$coefficients[1:dimen1])
-
     coef.table <- cbind(coef.p, coef.m, coef.sd, coef.se , coef.l , coef.u)
 
     if(hpd)
@@ -779,8 +1192,12 @@ function(fixed,
         dimnames(coef.table) <- list(names(coef.p), c("Mean", "Median", "Std. Dev.", "Naive Std.Error",
                 "95%CI-Low","95%CI-Upp"))
     }
-
     ans$prec<-coef.table
+
+    coef.table<-matrix(object$mc,nrow=1,ncol=5)
+    dimnames(coef.table) <- list(" ", c("Dbar", "Dhat", "pD", "DIC","LPML"))
+    ans$mc<-coef.table
+
     ans$nrec<-object$nrec
     ans$nsubject<-object$nsubject
 
@@ -798,6 +1215,10 @@ function(fixed,
     cat("Posterior Predictive Distributions (log):\n")	     
     print.default(format(summary(log(x$cpo)), digits = digits), print.gap = 2, 
             quote = FALSE) 
+
+    cat("\nModel's performance:\n")
+    print.default(format(x$mc, digits = digits), print.gap = 2, 
+    quote = FALSE)
             
     if (length(x$coefficients)) {
         cat("\nRegression coefficients:\n")
