@@ -2,7 +2,7 @@
 ### Fit a semiparametric aft model for interval censored data.
 ###
 ### Copyright: Alejandro Jara Vallejos, 2006
-### Last modification: 08-10-2006.
+### Last modification: 22-12-2006.
 ###
 ### This program is free software; you can redistribute it and/or modify
 ### it under the terms of the GNU General Public License as published by
@@ -40,8 +40,8 @@ function(formula,
          mcmc,
          state,
          status, 
-         data,
-         na.action)
+         data=sys.frame(sys.parent()),
+         na.action=na.fail)
 {
          #########################################################################################
          # call parameters
@@ -73,6 +73,50 @@ function(formula,
 	    type[y[,2]==-999]<-3
          }
 
+         #########################################################################################
+         # Elements for Pseudo Countour Probabilities' computation
+         #########################################################################################
+
+         tt<-terms(formula,data=data)
+         mat<-attr(tt,"factors")
+         namfact<-colnames(mat)
+         nvar<-dim(mat)[1]
+         nfact<-dim(mat)[2]
+         possiP<-matrix(0,ncol=2,nrow=nfact)
+         dataF<-model.frame(formula,data,xlev=NULL)
+         namD<-names(dataF)
+         isF<-sapply(dataF, function(x) is.factor(x) || is.logical(x))
+         nlevel<-rep(0,nvar)
+         for(i in 1:nvar)
+         {
+             if(isF[i])
+             {
+                nlevel[i]<-length(table(dataF[[i]]))
+             }
+             else
+             {
+                nlevel[i]<-1
+             }
+         }
+         startp<-1
+         for(i in 1:nfact)
+         {
+             tmp1<-1
+             for(j in 1:nvar)
+             {
+                 if(mat[j,i]==1 && isF[j])
+                 {
+                    tmp1<-tmp1*(nlevel[j]-1)
+                 }
+             }
+             endp<-startp+tmp1-1
+             possiP[i,1]<-startp    
+             possiP[i,2]<-endp
+             startp<-endp+1
+         }
+         dimnames(possiP)<-list(namfact,c("Start","End"))
+
+
          #########################################################################################
          # prior information
          #########################################################################################
@@ -167,13 +211,15 @@ function(formula,
          # working space
          #########################################################################################
 
+         extra<-1 
 	 maxint<-4*nrec+1
 	 maxend<-4*nrec
 	 ntotal<-nrec
-	 maxm<-log(0.0001)/log((ntotal+1)/(ntotal+2))
+	 maxm<-as.integer(log(0.0001)/log((ntotal+1)/(ntotal+2)))
 
  	 acrate<-0
 	 betac<-rep(0,p)
+	 clusts<-matrix(0,nrow=nrec,ncol=(nrec+1))	 
 	 cpo<-rep(0,nrec)
          etan<-rep(0,nrec)
 	 endp<-rep(0,maxend)
@@ -183,12 +229,14 @@ function(formula,
  	 imaxsc<-rep(0,nrec)
  	 imins<-rep(0,nrec)
  	 iminsc<-rep(0,nrec)
+ 	 index<-rep(0,maxm)	 
 	 intcount<-rep(0,maxint)
 	 intcount2<-rep(0,maxint)
 	 intind<-rep(0,(nrec+1))
 	 intind2<-rep(0,(nrec+1))
 	 intposso<-matrix(0,nrow=maxint,ncol=(nrec+1))
          intpossn<-matrix(0,nrow=maxint,ncol=(nrec+1))
+         limr<-matrix(0,nrow=nrec,ncol=2)         
          linfs<-rep(0,nrec)
          linfsc<-rep(0,nrec)
          lsups<-rep(0,nrec)
@@ -196,6 +244,7 @@ function(formula,
 	 mass<-rep(0,maxint)
 	 ncluster<-0
 	 prob<-rep(0,maxint)
+	 s<-rep(0,nrec)	 
 	 seed<-c(sample(1:29000,1),sample(1:29000,1),sample(1:29000,1))
 	 uvec<-rep(0,maxm)
 	 vvec<-rep(0,maxm)
@@ -213,7 +262,7 @@ function(formula,
          # calling the fortran code
          #########################################################################################
  
- 	 foo <- .Fortran("dpsurvint2",
+ 	 foo <- .Fortran("dpsurvint",
 	 	nrec      =as.integer(nrec),
 	 	p         =as.integer(p),
 		x         =as.double(x),	 	
@@ -222,6 +271,7 @@ function(formula,
 		mcmc      =as.integer(mcmcvec),
 		nsave     =as.integer(mcmc$nsave),
 		propv     =as.double(propv),
+		extra     =as.integer(extra),		
 		a0b0      =as.integer(a0b0),
 		betapm    =as.double(betapm),		
 		betapv    =as.double(betapv),		
@@ -241,23 +291,27 @@ function(formula,
  		maxint    =as.integer(maxint),
 		maxend    =as.integer(maxend),
 		maxm      =as.integer(maxm),
+		clusts    =as.integer(clusts),		
 		iflag     =as.integer(iflag),
 		imaxs     =as.integer(imaxs),
 		imaxsc    =as.integer(imaxsc),
 		imins     =as.integer(imins),
 		iminsc    =as.integer(iminsc),
+		index     =as.integer(index),				
 		intcount  =as.integer(intcount),
 		intcount2 =as.integer(intcount2),
 		intind    =as.integer(intind),
 		intind2   =as.integer(intind2),
 		intposso  =as.integer(intposso),
 		intpossn  =as.integer(intpossn),
+		s         =as.integer(s),		
                 seed      =as.integer(seed),
 		betac     =as.double(betac),
 		eta       =as.double(eta),
 		etan      =as.double(etan),
 		endp      =as.double(endp),
 		endp2     =as.double(endp2),
+		limr      =as.double(limr),		
                 linfs     =as.double(linfs),
                 linfsc    =as.double(linfsc),
                 lsups     =as.double(lsups),
@@ -267,7 +321,6 @@ function(formula,
                 uvec      =as.double(uvec),
                 vvec      =as.double(vvec),
 		vnew      =as.double(vnew),
-		vnew2     =as.double(vnew2),
 		wvec      =as.double(wvec),
 		workm1    =as.double(workm1),
 		workm2    =as.double(workm2),
@@ -314,7 +367,7 @@ function(formula,
 
 	 z<-list(modelname=model.name,coefficients=coeff,acrate=foo$acrate,call=cl,
 	         prior=prior,mcmc=mcmc,state=state,save.state=save.state,cpo=foo$cpo,
-	         nrec=nrec,p=p)
+	         nrec=nrec,p=p,possiP=possiP)
 	
 	 cat("\n\n")
 	 class(z)<-c("DPsurvint")
@@ -415,7 +468,21 @@ function(object,grid,xnew=NULL,hpd=TRUE, ...)
 			Cdelta<- sum(vwork>linf)
 			Cparam<- awork*plnorm(linf,meanlog=mwork,sdlog=swork,
 			                      lower.tail=FALSE,log.p=FALSE)
-			surv[j,k]<-(Cparam+Cdelta)/(awork+nrec)	                
+			tmp1<-(Cparam+Cdelta)
+			tmp2<-alpha[k]+nrec-tmp1
+
+			if(tmp2==0)
+			{
+			  surv[j,k]<-1
+			}
+			if(tmp1==0)
+			{
+			  surv[j,k]<-0
+			}
+			if(tmp1 != 0 && tmp2 != 0)
+			{
+			  surv[j,k]<-rbeta(n=1,shape1=tmp1,shape2=tmp2)
+			}  
 	          }
 	          pm[i,j]<-mean(surv[j,])
 		  pmed[i,j]<-median(surv[j,])
@@ -521,10 +588,237 @@ plot.predict.DPsurvint<-function(x,ask=TRUE,all=TRUE,band=FALSE,xlim=NULL,nfigr=
 
 
 ###
-### Tools for DPsurvint: print, summary, plot
+### Tools for DPsurvint: anova, print, summary, plot
 ###
 ### Copyright: Alejandro Jara Vallejos, 2006
-### Last modification: 08-10-2006.
+### Last modification: 15-12-2006.
+
+
+"anova.DPsurvint"<-function(object, ...)
+{
+
+######################################################################################
+cregion<-function(x,probs=c(0.90,0.975))
+######################################################################################
+#  Function to compute a simultaneous credible region for a vector 
+#  parameter from the MCMC sample
+# 
+#  Reference: Besag, J., Green, P., Higdon, D. and Mengersen, K. (1995)
+#             Bayesian computation and stochastic systems (with Discussion)
+#             Statistical Science, vol. 10, 3 - 66, page 30
+#  and        Held, L. (2004) Simultaneous inference in risk assessment; a Bayesian 
+#             perspective In: COMPSTAT 2004, Proceedings in Computational 
+#             Statistics (J. Antoch, Ed.) 213 - 222, page 214
+#
+#  Arguments 
+#  sample : a data frame or matrix with sampled values (one column = one parameter).
+#  probs  : probabilities for which the credible regions are computed.
+######################################################################################
+{
+    #Basic information
+     nmonte<-dim(x)[1]
+     p<-dim(x)[2]
+     
+    #Ranks for each component
+     ranks <- apply(x, 2, rank, ties.method="first")
+     
+    #Compute the set S={max(nmonte+1-min r_i(t) , max r_i(t)): t=1,..,nmonte}
+     left <- nmonte + 1 - apply(ranks, 1, min)
+     right <- apply(ranks, 1, max)
+     S <- apply(cbind(left, right), 1, max)
+     S <- S[order(S)]
+    
+    #Compute the credible region
+     k <- floor(nmonte*probs)     
+     tstar <- S[k]
+     out<-list()
+     for(i in 1:length(tstar))
+     {
+        upelim <- x[ranks == tstar[i]]
+        lowlim <- x[ranks == nmonte + 1 - tstar[i]]    
+        out[[i]] <- rbind(lowlim, upelim)
+        rownames(out[[i]]) <- c("Lower", "Upper")
+        colnames(out[[i]]) <- colnames(x)
+     }
+     names(out) <- paste(probs)
+     return(out)
+}
+
+######################################################################################
+cint<-function(x,probs=c(0.90,0.975))
+######################################################################################
+#  Function to compute a credible interval from the MCMC sample
+#
+#  Arguments 
+#  sample : a data frame or matrix with sampled values (one column = one parameter).
+#  probs  : probabilities for which the credible regions are to be computed.
+######################################################################################
+{
+    #Compute the credible interval
+     delta<-(1-probs)/2
+     lprobs<-cbind(delta,probs+delta) 
+     out<-matrix(quantile(x,probs=lprobs),ncol=2)
+     colnames(out) <- c("Lower","Upper")
+     rownames(out) <- paste(probs)
+     return(out)
+}
+
+######################################################################################
+hnulleval<-function(mat,hnull)
+######################################################################################
+#  Evaluate H0
+#  AJV, 2006
+######################################################################################
+{
+     npar<-dim(mat)[2]   
+     lower<-rep(0,npar)
+     upper<-rep(0,npar)
+     for(i in 1:npar)
+     {
+        lower[i]<-mat[1,i]< hnull[i]
+        upper[i]<-mat[2,i]> hnull[i]
+     }
+     total<-lower+upper
+     out<-(sum(total==2) == npar)
+     return(out)
+}
+
+######################################################################################
+hnulleval2<-function(vec,hnull)
+######################################################################################
+#  Evaluate H0
+#  AJV, 2006
+######################################################################################
+{
+     lower<-vec[1]< hnull
+     upper<-vec[2]> hnull
+
+     total<-lower+upper
+     out<-(total==2)
+     return(out)
+}
+
+
+######################################################################################
+pcp<-function(x,hnull=NULL,precision=0.001,prob=0.95,digits=digits)
+######################################################################################
+#  Function to compute Pseudo Countour Probabilities (Region)
+#  AJV, 2006
+######################################################################################
+{
+    if(is.null(hnull))hnull<-rep(0,dim(x)[2])
+    if (dim(x)[2]!=length(hnull)) stop("Dimension of x and hnull must be equal!!")
+
+    probs <- seq(precision, 1-precision, by=precision)
+    neval <- length(probs)
+    probsf <- c(prob,probs)
+    cr <-  cregion(x,probs=probsf)
+
+    is.hnull <- hnulleval(cr[[2]],hnull)
+    if(is.hnull)
+    {
+       pval <- 1-precision
+    }   
+    else
+    {
+       is.hnull <- hnulleval(cr[[length(cr)]],hnull)
+       if (!is.hnull) 
+       {
+         pval <- precision
+       }  
+       else
+       {
+         is.hnull<-rep(0,neval+1)
+         for(i in 1:(neval+1))
+         {
+            is.hnull[i] <- hnulleval(cr[[i]],hnull)
+         }   
+         is.hnull <- is.hnull[-1]
+         first <- neval - sum(is.hnull) + 1
+         pval <- 1 - probs[first]
+       }
+    }
+    output <- list(cr=cr[[1]], prob=prob, pval=pval,hnull=hnull)
+    return(output)
+}
+
+
+######################################################################################
+pcp2<-function(x,hnull=NULL,precision=0.001,prob=0.95)
+######################################################################################
+#  Function to compute Pseudo Countour Probabilities (Interval)
+#  AJV, 2006
+######################################################################################
+{
+    if(is.null(hnull))hnull<-0
+    probs <- seq(precision, 1-precision, by=precision)
+    neval <- length(probs)
+    probsf <- c(prob,probs)
+    cr <-  cint(x,probs=probsf)
+
+    is.hnull <- hnulleval2(cr[2,],hnull)
+    if(is.hnull)
+    {
+       pval <- 1-precision
+    }   
+    else
+    {
+       is.hnull <- hnulleval2(cr[(neval+1),],hnull)
+       if (!is.hnull) 
+       {
+         pval <- precision
+       }  
+       else
+       {
+         is.hnull<-rep(0,neval+1)
+         for(i in 1:(neval+1))
+         {
+            is.hnull[i] <- hnulleval2(cr[i,],hnull)
+         }   
+         is.hnull <- is.hnull[-1]
+         first <- neval - sum(is.hnull) + 1
+         pval <- 1-probs[first]
+       }
+    }
+    output <- list(cr=cr[1,], prob=prob, pval=pval,hnull=hnull)
+    return(output)
+}
+
+######################################################################################
+######################################################################################
+######################################################################################
+
+    possiP<-object$possiP
+    nfact<-dim(possiP)[1]
+    P<-rep(0,nfact)
+    df<-rep(0,nfact)
+    
+    for(i in 1:nfact)
+    {
+        df[i]<-1
+        if((possiP[i,2]-possiP[i,1])>0)
+        { 
+           x<-matrix(object$save.state$thetasave[,possiP[i,1]:possiP[i,2]])
+           foo<-pcp(x=x) 
+           P[i]<-foo$pval
+           df[i]<-(possiP[i,2]-possiP[i,1])+1
+        }
+        else
+        {
+           x<-object$save.state$thetasave[,possiP[i,1]:possiP[i,2]]
+           foo<-pcp2(x=x) 
+           P[i]<-foo$pval
+        }
+    }
+
+    table <- data.frame(df,P) 
+    dimnames(table) <- list(rownames(possiP), c("Df","PsCP"))
+    structure(table, heading = c("Table of Pseudo Contour Probabilities\n", 
+        paste("Response:", deparse(formula(object)[[2]]))), class = c("anovaPsCP",
+        "data.frame"))
+}
+
+
 
 "print.DPsurvint"<-function (x, digits = max(3, getOption("digits") - 3), ...) 
 {
